package com.app;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.io.ClassPathResource;
import org.springframework.http.ResponseEntity;
import org.springframework.http.ResponseEntity.BodyBuilder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.constraints.Max;

/**
 * @author Sarvesh Kumar
 * Provide update and download server in the application.
 */

@RestController
public class UploadDownloadController {
    private static final Logger logger = LoggerFactory.getLogger(UploadDownloadController.class);
    static String topCustomers[] = new String[4];
    static String topProducts[] = new String[4];
    static String topCustomerProductDetails[] = new String[4];

    @GetMapping({"/download/{fileName:.+}"})
    public String downloadData(@PathVariable String fileName) {
        String content="";
        try {
            System.out.println("-->> Download Data FileName: "+fileName);
            InputStream resource1 = new ClassPathResource("data/"+fileName).getInputStream();
            BufferedReader reader = new BufferedReader(new InputStreamReader(resource1));
            content = reader.lines().collect(Collectors.joining("\n"));
        } catch (Exception ex) {
            logger.error(ex.getMessage());
        }
        return content;
    }

    @PostMapping({"/upload/{type:.+}/{number:.+}"})
    public void UploadData(@PathVariable String type, @PathVariable int number, @RequestParam MultipartFile file) {
        try {
            System.out.println("-->> UploadData Type: "+type+" | Number: "+number);
            InputStream inputStream = file.getInputStream();
            BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(inputStream));

            StringBuffer lines = new StringBuffer("");
            lines.append(bufferedReader.readLine());
            for(String line = bufferedReader.readLine(); line != null; line = bufferedReader.readLine()) {
                lines.append("\n"+line);
            }
            bufferedReader.close();

            switch (type){
                case "topCustomers":    topCustomers[number] = lines.toString(); break;
                case "topProducts":     topProducts[number] = lines.toString(); break;
                case "topCustomerProductDetails":   topCustomerProductDetails[number] = lines.toString(); break;
            }

        } catch (Exception ex) {
            logger.error(ex.getMessage());
        }
    }

    @GetMapping({"/data/{type:.+}/{number:.+}"})
    public String getData(@PathVariable String type, @PathVariable int number) {
        System.out.println("-->> GetData Type: "+type+" | Number: "+number);
        String result = "";
        try {
            switch (type) {
                case "topCustomers":    result = topCustomers[number]; break;
                case "topProducts":     result = topProducts[number]; break;
                case "topCustomerProductDetails":   result = topCustomerProductDetails[number]; break;
            }
        } catch (Exception ex) { }
        return result!=null ? result.replace("(","").replace(")","") : null;
    }

}
